/**
 * Person in einem Kreis.
 * 
 * @author Owyn
 * @version 2008-05-08
 */
class Person {
    
    private int number; // Nummer der Person
    private Person next; // Nachfolger im Kreis
    
    /**
     * Erzeugt eine neue Person mit Nachfolger.
     * 
     * @param number Nummer der Person
     * @param next Verweis auf den Nachfolger oder null
     */
    Person(int number, Person next) {
        this.number = number;
        this.next = next;
    }
    
    /**
     * &Auml;ndert den Nachfolger der Person.
     * 
     * @param next Der neue Nachfolger
     */
    public void setNext(Person next) {
        this.next = next;
    }
    
    /**
     * @return number Nummer der Person
     */
    public int number() { return number; }

    /**
     * @return number Nachfolger der Person
     */
    public Person next() { return next; }
    
}

/**
 * Kreis aus Personen (zirkul&auml;re Liste).
 * 
 * @author Owyn
 * @version 2008-05-16
 */
public class Circle {
    
    Person start; // Startperson beim Abzaehlen
    
    /**
     * Erzeugt einen leeren Kreis.
     */
    Circle() {
        start = null;
    }
    
    /**
     * Testet, ob der Kreis leer ist.
     * 
     * @return true, wenn der Kreis leer ist
     */
    public boolean isEmpty() {
        return start == null;
    }
    
    /**
     * Gibt eine Textdarstellung des Kreises zur&uuml;ck.
     * 
     * @return Textdarstellung im Format nr1 nr2 ... nrN
     */
    public String toString() {
        if (isEmpty()) return ""; // Kreis leer? Direkt aufhoeren.
        String ret = "" + start.number();
        Person current = start; // Start beim Anfang
        while (current.next() != start) { // Solange wir nicht wieder beim Anfang sind...
            current = current.next(); // ...die naechste Person...
            ret += (" " + current.number()); // ...an den String anhaengen.
        }
        return ret;
    }
    
    /**
     * F&uuml;gt eine neue Person vor der Startperson in den Kreis ein.
     * 
     * @param number Die Nummer der neuen Person
     */
    public void insert(int number) {
        if (isEmpty()) { // Kreis leer?
            start = new Person(number, null); // Person einfuegen...
            start.setNext(start); // ...die sich selbst als Nachfolger hat.
        } else {
            Person current = start; // Vom Start...
            while (current.next() != start) current = current.next(); //...bis direkt vor dem Start abzaehlen
            current.setNext(new Person(number, start)); // Neue Person zwischen letzter und erster Person einfuegen
        }
    }

    /**
     * Entfernt eine Person an einer abgez&auml;hlten Stelle im Kreis und setzt
     * den Start des Kreises auf den Nachfolger der entfernten Person.
     * 
     * @param pos Die Position der zu entfernenden Position ausgehend vom Start
     * @return Die Nummer der entfernten Person
     */
    public int remove (int pos) {
        if (isEmpty()) return 0; // Kreis leer? Direkt aufhoeren.
        int removed = 0;
        Person current = start; // Start beim Anfang
        for (int i = 1; i < pos - 1; i++) current = current.next(); // gehe zur Person vor der abgezaehlten
        removed = current.next().number(); // Nummer der abgezaehlten Person merken
        if(current == current.next()) { // Nur eine Person im Kreis:
            start = null; // Kreis ist jetzt leer.
        } else { // Kreis enthaelt mehr als eine Person:
            current.setNext(current.next().next()); // Die abgezaehlte Person wird aus dem Kreis genommen
            start = current.next(); // Der Kreis startet jetzt beim Nachfolger der abgezaehlten Person
            
        }
        return removed;
    }
    
}

